"""
InfraHouse Core Library.

This library provides reusable classes and functions for AWS and GitHub integration.

Main modules:
    - aws: AWS service integration (EC2, DynamoDB, Secrets Manager, Route53)
    - github: GitHub Actions and API integration
    - logging: Logging configuration utilities
    - timeout: Timeout context manager for operations
"""

__author__ = """Oleksandr Kuzminskyi"""
__email__ = "aleks@infrahouse.com"
__version__ = "0.27.0"
